from datetime import datetime
import logging
import src.database.connection as database
import src.database.utils as utils
from src.database.queries import IntegrationType
from src.jobs.system_jobs import OnlineLogger
from src.database.utils import DatabaseConfig
import src.api.okinghub as api_okHUB
from src.api import slack
from src.database import queries
import src
import time

logger = logging.getLogger()
send_log = OnlineLogger.send_log


def job_send_clients_colect(job_config_dict: dict):
    """
    Job para enviar clientes
    Args:
        job_config_dict: Configuração do job
    """
    try:
        db_config = utils.get_database_config(job_config_dict)
        if db_config.sql is None:
            send_log(job_config_dict.get('job_name'), job_config_dict.get('enviar_logs'), False,
                     f'Comando sql para inserir clientes no semaforo nao encontrado', 'warning', 'CLIENTE')
        else:
            clients = query_clients_erp(job_config_dict, db_config)
            if clients is not None and len(clients) > 0:
                send_log(job_config_dict.get('job_name'), job_config_dict.get('enviar_logs'), False,
                         f'Clientes para atualizar {len(clients)}', 'info', 'CLIENTE')

                api_clients = []
                for client in clients:
                    if api_clients.__len__() < 50:
                        api_clients.append(client)
                    else:
                        send_log(job_config_dict.get('job_name'), False, False,
                                 f'Enviando Pacote: {api_clients.__len__()}', 'info', 'CLIENTE', '')
                        response = api_okHUB.post_clients(api_clients)
                        api_clients = []
                        send_log(job_config_dict.get('job_name'), False, False, f'Tratando retorno', 'info', 'CLIENTE',
                                 '')
                        validade_response_client(response, db_config, job_config_dict)

                # Se ficou algum sem processa
                if api_clients.__len__() > 0:
                    send_log(job_config_dict.get('job_name'), False, False, f'Enviando Pacote: {api_clients.__len__()}',
                             'info', 'CLIENTE', '')
                    response = api_okHUB.post_clients(api_clients)
                    api_clients = []
                    send_log(job_config_dict.get('job_name'), False, False, f'Tratando retorno', 'info', 'CLIENTE', '')
                    validade_response_client(response, db_config, job_config_dict)
            else:
                send_log(job_config_dict.get('job_name'), job_config_dict.get('enviar_logs'), False,
                         f'Nao existem clientes a serem enviados no momento', 'warning', 'CLIENTE')
    except Exception as e:
        send_log(job_config_dict.get('job_name'), job_config_dict.get('enviar_logs'), False,
                 f'Erro durante execucao do job: {str(e)}', 'error', 'CLIENTE')


def query_clients_erp(job_config_dict: dict, db_config: DatabaseConfig):
    """
    Consulta os clientes para atualizar no banco de dados
    Args:
        job_config_dict: Configuração do job
        db_config: Configuracao do banco de dados

    Returns:
        Lista de clientes para atualizar
    """
    db = database.Connection(db_config)
    conn = db.get_conect()
    cursor = conn.cursor()

    try:
        # monta query com EXISTS e NOT EXISTS
        # verificar se já possui WHERE
        newsql = utils.final_query(db_config)
        cursor.execute(newsql)
        rows = cursor.fetchall()
        columns = [col[0] for col in cursor.description]
        results = list(dict(zip(columns, row)) for row in rows)
        cursor.close()
        conn.close()

        clients = []
        if len(results) > 0:
            clients = client_dict(results)
        return clients
    except Exception as ex:
        if src.exibir_interface_grafica:
            raise
        else:
            send_log(job_config_dict.get('job_name'), job_config_dict.get('enviar_logs'), False,
                     f' Erro ao consultar clientes no banco semaforo: {str(ex)}', 'error', 'CLIENTE')


def client_dict(clients):
    lista = []
    for row in clients:
        pdict = {
            'token': str(src.client_data.get('token_oking')),
            'kdm': str(row['kdm']),
            'agente': str(row['agente']),
            'unidade': str(row['unidade']),
            'codigo_cliente': str(row['codigo_cliente']),
            'nome': str(row['nome']),
            'cnpj': str(row['cnpj']),
            'data_cadastro': str(row['data_cadastro']),
            'ddd': int(row['ddd']),
            'telefone': str(row['telefone']),
            'email': str(row['email']),
            'endereco': str(row['endereco']),
            'numero': int(row['numero']),
            'bairro': str(row['bairro']),
            'uf': str(row['uf']),
            'faturamento_anual_total': int(row['faturamento_anual_total']),
            'consumo_cimento': float(row['consumo_cimento']),
            'percentual_faturamento_cimento': int(row['percentual_faturamento_cimento']),
            'canal': str(row['canal']),
            'municipio': str(row['municipio']),
            'uf_municipio': str(row['uf_municipio']),
            'codigo_ibge': str(row['codigo_ibge']),
            "cep": str(row['cep']),
            "contato": str(row['contato']),
            "contato_cargo": str(row['contato_cargo']),
            "canal_segmento": str(row['canal_segmento']),
            "status": str(row['status']),
            "vendedor": str(row['vendedor']),
            "ultima_atualizacao": None,
            "segmento_cliente": str(row['segmento_cliente']),
            "marca_exclusivo": str(row['marca_exclusivo']),
            "multimarca_percentual": int(row['multimarca_percentual']),
            "marca_principal_concorrente": str(row['marca_principal_concorrente']),
            "marca_segundo_concorrente": str(row['marca_segundo_concorrente']),
        }
        lista.append(pdict)

    return lista


def validade_response_client(response, db_config, job_config_dict):
    if response is not None:
        conexao = database.Connection(db_config)
        conn = conexao.get_conect()
        cursor = conn.cursor()

        # Percorre todos os registros
        for item in response:
            identificador = item.identificador
            identificador2 = item.identificador2
            if item.sucesso == 1 or item.sucesso == 'true':
                try:
                    cursor.execute(queries.get_insert_update_semaphore_command(db_config.db_type),
                                   queries.get_command_parameter(db_config.db_type,
                                                                 [identificador, identificador2,
                                                                  IntegrationType.COLETA_DADOS_CLIENTE.value,
                                                                  'SUCESSO']))
                    # atualizados.append(response['identificador'])
                except Exception as e:
                    send_log(job_config_dict.get('job_name')
                             , job_config_dict.get('enviar_logs')
                             , True
                             , f'Erro ao atualizar Cliente do sku: {item.identificador}, Erro: {str(e)}'
                             , 'error'
                             , 'CLIENTE'
                             , f'{item.identificador}-{item.identificador2}')
            else:
                send_log(job_config_dict.get('job_name'), job_config_dict.get('enviar_logs'), True,
                         f'Erro ao atualizar Cliente para o sku: {identificador}, {item.Message}', 'warning',
                         'CLIENTE', identificador)

        cursor.close()
        conn.commit()
        conn.close()
